//-------------------------------------------------------BEGINHEADER--
// This file is part of LoopSim (arXiv:1006.2144).
//
// Copyright 2009-2011 Mathieu Rubin, Gavin P. Salam and Sebastian Sapeta
//
// This software is provided as a snapshot of the ongoing LoopSim
// research project. As such, you may not redistribute it without the
// authors' permission.
//
// If you use LoopSim as part of your scientific work, you should
// discuss and agree with the LoopSim authors how best to acknowledge
// LoopSim in your work (e.g. whether through a reference, or through
// joint authorship with the LoopSim authors). 
//
// To help guide LoopSim's proper use in its current early stage of
// development, a condition of use of LoopSim is that any results that
// you obtain with it must be shown to the LoopSim authors before
// they are made public.
//
// $Id: ExtractLHEvent.hh 1919 2013-06-29 15:25:14Z sapeta $
//-------------------------------------------------------ENDHEADER----

#ifndef __EXTRACTLHEVENT_HH__
#define __EXTRACTLHEVENT_HH__
#include <iostream>      
#include <vector>
#include <string>
#include <sstream>
#include <memory>
#include "LoopSim.hh"
#include "InterfaceCommon.hh"
#include "ExtractEventBase.hh"

using namespace std;
using namespace fastjet;

/// enum for available generators which produced the lhef 
enum Generator{
  MCFM, VBFNLO, NONE
};


/// enum specifying exact form of LHEF input
///
enum LHEFormat {
  with_extra_info, /// default format with used with mcfm
  vbfnlo           /// vbfnlo fromat without optional line 
};


/// Class for reading LHEF input and transforming events into LS Event objects.
/// It can handle processes with Z/W^+/W^- and/or partons.
class ExtractLHEvent : public ExtractEventBase {
public:
  // constructor
  ExtractLHEvent();
  /// read each line and take action depending on its content
  void readline(string line, LHEFormat lheformat = with_extra_info);
  /// get pinter to next event if available, otherwise return zero
  Event* next_event() { return _current_event.get(); }
  EventExtraInfo current_event_extra_info() const {
      return _current_event_extra_info; } 
  /// number of the current event
  unsigned int iev() const { return (unsigned int) _ientry; }
  /// number of the current iteration
  int it() const { return _it; }
  /// true if we have just read all events from a single iteration
  bool periodic_output();
  /// true if we got the end of LHEF
  bool end_of_file() const {return _end_of_file;}
  void throw_error_message(string message, string explanation = "");
  /// returns info about generator that produced LHEF
  string generator() const; 
  private:
  /// generator which produced the LHEF
  Generator _generator;
  /// true if we are currently reading an event from the input
  bool _reading_event_block;
  stringstream _current_event_block;
  /// auto pointer to the current event; if zero, that means that we are 
  /// in the process of reading next event from the input
  auto_ptr<Event> _current_event;
  /// analyse functions, two different versions which should 
  /// be combined into one soon
  void _analyse_event();
  void _analyse_event_vbfnlo();
  void _check_boson(int flav, vector<PseudoJet>& lepton1,
                              vector<PseudoJet>& lepton2);
  // extra info about the current event
  EventExtraInfo _current_event_extra_info;
  int _it;            // number of current iteration
  int _ientry;        // just an event counter
  bool _end_of_file;

  // 
  int _n_unused;
  int _n_bosons;
  int _n_leptons;
};

#endif // __EXTRACTLHEVENT_H__
