//-------------------------------------------------------BEGINHEADER--
// This file is part of LoopSim (arXiv:1006.2144).
//
// Copyright 2009-2011 Mathieu Rubin, Gavin P. Salam and Sebastian Sapeta
//
// This software is provided as a snapshot of the ongoing LoopSim
// research project. As such, you may not redistribute it without the
// authors' permission.
//
// If you use LoopSim as part of your scientific work, you should
// discuss and agree with the LoopSim authors how best to acknowledge
// LoopSim in your work (e.g. whether through a reference, or through
// joint authorship with the LoopSim authors). 
//
// To help guide LoopSim's proper use in its current early stage of
// development, a condition of use of LoopSim is that any results that
// you obtain with it must be shown to the LoopSim authors before
// they are made public.
//
// $Id: InterfaceCommon.hh 1919 2013-06-29 15:25:14Z sapeta $
//-------------------------------------------------------ENDHEADER----


#ifndef __INTERFACECOMMON_HH__
#define __INTERFACECOMMON_HH__
#include <iostream>      
#include <vector>
#include <string>
#include <sstream>
#include <memory>
#include "LoopSim.hh"

using namespace std;
using namespace fastjet;

/// Class storing extra information about the event that do not go 
/// into Event object from LoopSim library
///
class EventExtraInfo {
  public:
  EventExtraInfo() {}
  void set_part(string part) {_part = part;}
  void set_alphapower(int alphapower) {_alphapower = alphapower;}
  void set_nd(int nd) { _nd = nd; }
  string part() const { return _part;}
  int alphapower() const { return _alphapower;}
  int nd() const { return _nd; }
  bool is_lord () const { return (_part == "lord" || _part == "B"); }
  /// to have a consistent approach both for LH and ntuples we adopt 
  /// the convention that dipole event is also marked as real 
  bool is_real () const { return (_part == "real" || _part == "R" ||
                                  _part == "I" || _part == "D" ); }
  bool is_virt () const { return (_part == "virt" || _part == "L"  || 
                                  _part == "V"); }
  bool is_a_dipole() const {
    return ((_part == "real" && _nd > 0) || _part == "I" || _part == "D");
  }	  
  void get_order_and_nloops(int& order, int& nloops) const {
    if (this->is_lord()) {
      order = 0; nloops = 0;
    } else if (this->is_virt()) {
      order = 1; nloops = 1;
    } else if (this->is_real()) {
      order = 1;
      if(this->is_a_dipole()) nloops = 1;
      else nloops = 0;
    }
  }
  private:
  string _part;    // order: lord, virt, real
  int _alphapower; 
  int _nd;         // dipole number: 0 (lord, virt, real), > 0 (dipoles)


};

#endif // __INTERFACECOMMON_H__
