//-------------------------------------------------------BEGINHEADER--
// This file is part of LoopSim (arXiv:1006.2144).
//
// Copyright 2009-2011 Mathieu Rubin, Gavin P. Salam and Sebastian Sapeta
//
// This software is provided as a snapshot of the ongoing LoopSim
// research project. As such, you may not redistribute it without the
// authors' permission.
//
// If you use LoopSim as part of your scientific work, you should
// discuss and agree with the LoopSim authors how best to acknowledge
// LoopSim in your work (e.g. whether through a reference, or through
// joint authorship with the LoopSim authors). 
//
// To help guide LoopSim's proper use in its current early stage of
// development, a condition of use of LoopSim is that any results that
// you obtain with it must be shown to the LoopSim authors before
// they are made public.
//
// $Id: LSAnalysisBase.hh 1999 2013-08-22 14:21:20Z sapeta $
//-------------------------------------------------------ENDHEADER----

// todo:
//   * there is a potential problem in description() when the pointer _jet_def
//      is not defined

#ifndef __LSANALYSISBASE_HH__
#define __LSANALYSISBASE_HH__
#include <iostream>      
#include "fastjet/PseudoJet.hh"
#include "fastjet/ClusterSequenceArea.hh"
#include "SimpleHist.hh"
#include "SimpleHist2D.hh"
#include "LoopSim.hh"
#include "PresInsOrderMap.hh"

using namespace std;
using namespace fastjet;

/// Base class for LoopSim analysis.
///
/// It takes events in the form of LS Event class, one by one, and analyses them
/// by applying user defined cuts and fills user defined histograms
///  
/// When requested, it writes results to an output file.
///
class LSAnalysisBase {
  public:
  /// constructor;
  /// user is supposed to define jet algorithm, cuts and declare histograms
  LSAnalysisBase() : _jet_def(0) {}
  virtual ~LSAnalysisBase() {}
  /// get an event, apply  cuts and fill histograms
  virtual void analyse(const Event& ls_evt, const bool orig) {
    cerr << "Error in LSAnalysisBase: "
         << "The function analyse(const Event& ls_evt, const bool orig) "
	 << "has not been overridden by the derived class!" << endl;
    exit(1);
  }
  virtual double HT(const vector<PseudoJet>& jets);
  /// write histograms to file
  ///   \param int iev              number of current event
  ///   \param int iit              number of already finished iterations
  ///   \param string description   description going to top of the file
  ///   \param string outfilename   name of the output file
  virtual void write(int iev, int iit, string header, string outfilename);
  virtual string description();
  static int iev_orig;
  protected:
  map<string,SimpleHist> _histograms;
  PresInsOrderMap<string,SimpleHist> _ordered_histograms;
  map<string,SimpleHist2D> _histograms2d;
  map<string,double> _cuts;
  JetDefinition *_jet_def;
  virtual void _set_no_cuts(); 
  virtual void _something_went_wrong(string where);

};
#endif // __LSANALYSISBASE_H__
