//-------------------------------------------------------BEGINHEADER--
// This file is part of LoopSim (arXiv:1006.2144).
//
// Copyright 2009-2011 Mathieu Rubin, Gavin P. Salam and Sebastian Sapeta
//
// This software is provided as a snapshot of the ongoing LoopSim
// research project. As such, you may not redistribute it without the
// authors' permission.
//
// If you use LoopSim as part of your scientific work, you should
// discuss and agree with the LoopSim authors how best to acknowledge
// LoopSim in your work (e.g. whether through a reference, or through
// joint authorship with the LoopSim authors). 
//
// To help guide LoopSim's proper use in its current early stage of
// development, a condition of use of LoopSim is that any results that
// you obtain with it must be shown to the LoopSim authors before
// they are made public.
//
// $Id: LoopSimDriver.hh 1919 2013-06-29 15:25:14Z sapeta $
//-------------------------------------------------------ENDHEADER----


#ifndef __LOOPSIMDRIVER_HH__
#define __LOOPSIMDRIVER_HH__
#include <iostream>      
#include <fstream>      
#include <string>
#include <map>
#include "LoopSim.hh"
#include "ExtractLHEvent.hh"
#include "InputDat.hh"
#include "LSAnalysisBase.hh"

#ifdef NTUPLEROOT
#include "ExtractNtupleEvent.hh"
#endif //NTUPLEROOT

using namespace std;

/// enum for available generators which produced the lhef 
enum EventSource{
  LHE, NTUPLE, NOSOURCE
};


/// Class for taking care of reading input events, gerating LoopSim events and
/// analysising them and generating output.
class LoopSimDriver {
  public:
  // constructor
  LoopSimDriver(int argc, char ** argv);
  LoopSimDriver(int argc, char ** argv, InputDat inputdat);
  ~LoopSimDriver();
  // functions for adding analyses 
  void add_analysisRmin(LSAnalysisBase* lsa, const string& name = "");
  void add_analysisRmax(LSAnalysisBase* lsa, const string& name = "");
  void add_analysisRmid(LSAnalysisBase* lsa, const string& name = "");
  /// main function doing all the looping over events and analyses
  void run();
  bool process_next_event(ExtractEventBase& record);
  bool do_cuts() const {return _do_cuts;}
  int nborn() const {return _nborn;}
  bool jet_veto() const {return _jet_veto;}
  bool jet_veto2nd() const {return _jet_veto2nd;}
  string event_source() const;
  private:
  InputDat* _inputdat;
  // maps storing the analyses for each RLS and corresponding
  // output file names
  map<string,LSAnalysisBase*> _lsanalysesRmid;
  map<string,LSAnalysisBase*> _lsanalysesRmax;
  map<string,LSAnalysisBase*> _lsanalysesRmin;
  map<string,string> _outfilenameRmin;
  map<string,string> _outfilenameRmax;
  map<string,string> _outfilenameRmid;
  // variables extracted from card file
  string _outfilename;
  string _generator;
  string _ntuple_name;
  EventSource _event_source;
  void _set_mandatory_options();
  void _check_event_source();
  bool _file_exists(string _file_name) const;
  int _nborn;
  int _verbose;
  bool _spread_virtuals;
  bool _make_loopsim_analysis;
  bool _do_cuts;
  bool _jet_veto;
  bool _jet_veto2nd;
  double _scale_factor;
  double _RLS;
  double _dR;
  size_t _datpos;

};
#endif // __LOOPSIMDRIVER_H__
